"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vsls = require("vsls/vscode");
const redux_1 = require("redux");
const sharedState_1 = require("./sharedState");
const constants_1 = require("./constants");
function setInitialState(initialState) {
    return {
        type: constants_1.SET_INITIAL_STATE_ACTION_NAME,
        initialState,
    };
}
exports.vslsStoreEnhancer = (actionFilter = (action) => true) => (next) => (reducer, initialStateOrEnhancer, enhancer) => {
    const vslsRedux = new VSLSRedux();
    const middlewareEnhancer = redux_1.applyMiddleware(vslsRedux.createMiddleware());
    let store;
    if (typeof initialStateOrEnhancer === 'object') {
        // DeepPartial<any>
        const composedEnhancer = enhancer
            ? redux_1.compose(enhancer, middlewareEnhancer)
            : middlewareEnhancer;
        store = next(reducer, initialStateOrEnhancer, composedEnhancer);
    }
    else {
        // StoreEnhancer
        const composedEnhancer = initialStateOrEnhancer
            ? redux_1.compose(initialStateOrEnhancer, middlewareEnhancer)
            : middlewareEnhancer;
        store = next(reducer, composedEnhancer);
    }
    vslsRedux.setStore(store);
    return store;
};
class VSLSRedux {
    constructor() {
        this.remoteService = null;
        this.store = null;
        this.defaultActionFilter = (action) => {
            return action.type !== constants_1.SET_INITIAL_STATE_ACTION_NAME;
        };
        this.init();
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            const vslsAPI = yield vsls.getApiAsync();
            if (vslsAPI) {
                vslsAPI.onDidChangeSession((e) => {
                    const isSessionEnded = !e.session.id;
                    if (isSessionEnded) {
                        this.remoteService = null;
                        return;
                    }
                    switch (e.session.role) {
                        case vsls.Role.Guest:
                            this.remoteService = new GuestService(this.store);
                            break;
                        case vsls.Role.Host:
                            this.remoteService = new HostService(this.store);
                            break;
                        default:
                            return;
                    }
                });
            }
        });
    }
    createMiddleware(actionFilter = (action) => true) {
        return () => {
            return (next) => (action) => __awaiter(this, void 0, void 0, function* () {
                const syncAction = this.defaultActionFilter(action) && actionFilter(action);
                if (syncAction && this.remoteService) {
                    this.remoteService.middleware()(next)(action);
                }
                else {
                    next(action);
                }
            });
        };
    }
    setStore(store) {
        this.store = store;
    }
}
/**
 * The HostService sends the host state to any guests that request it via the getState request
 * and will stamp the vslsId onto any incoming actions and redistribute them to all guests.
 */
class HostService {
    constructor(store) {
        this.service = null;
        this.nextActionId = 0;
        this.middleware = () => {
            return (next) => (action) => {
                action.vslsId = this.nextActionId++;
                next(action);
                this.service.notify(constants_1.DISPATCH_NOTIFICATION, action);
            };
        };
        this.init(store);
    }
    init(store) {
        return __awaiter(this, void 0, void 0, function* () {
            const vslsAPI = yield vsls.getApiAsync();
            if (vslsAPI) {
                // TODO: Namespace this service name by the calling extension's name
                this.service = yield vslsAPI.shareService(constants_1.SERVICE_NAME_SUFFIX);
                if (this.service.isServiceAvailable) {
                    this.service.onRequest(constants_1.GET_STATE_REQUEST, (args) => {
                        const initialState = {
                            state: sharedState_1.getSharedState(),
                            nextActionId: this.nextActionId,
                        };
                        return initialState;
                    });
                    this.service.onNotify(constants_1.DISPATCH_NOTIFICATION, (args) => {
                        store.dispatch(args);
                    });
                }
                else {
                    // Failed to share service
                    // TODO: Handle this
                }
            }
        });
    }
}
/**
 * The GuestService requests the intial state from the host, forwards all actions to the host,
 * and dispatches all actions received by the host in order.
 */
class GuestService {
    constructor(store) {
        this.serviceProxy = null;
        this.awaitedActions = {};
        this.nextActionId = 0;
        this.middleware = () => {
            return (next) => (action) => __awaiter(this, void 0, void 0, function* () {
                if (typeof action.vslsId === 'undefined') {
                    this.serviceProxy.notify(constants_1.DISPATCH_NOTIFICATION, action);
                }
                else {
                    if (action.vslsId !== this.nextActionId) {
                        yield new Promise((resolve, reject) => {
                            this.awaitedActions[action.vslsId || 0] = resolve;
                        });
                    }
                    next(action);
                    this.nextActionId++;
                    if (this.awaitedActions[this.nextActionId]) {
                        this.awaitedActions[this.nextActionId]();
                        delete this.awaitedActions[this.nextActionId];
                    }
                }
            });
        };
        this.store = store;
        this.init(store);
    }
    init(store) {
        return __awaiter(this, void 0, void 0, function* () {
            const vslsAPI = yield vsls.getApiAsync();
            if (vslsAPI) {
                // TODO: Namespace this service name by the calling extension's name
                this.serviceProxy = yield vslsAPI.getSharedService(constants_1.SERVICE_NAME_SUFFIX);
                if (this.serviceProxy.isServiceAvailable) {
                    this.serviceProxy.onNotify(constants_1.DISPATCH_NOTIFICATION, (args) => {
                        store.dispatch(args);
                    });
                    this.getInitialState();
                }
                else {
                    // Service not available. Not installed on host?
                    // TODO: Handle this
                }
            }
        });
    }
    getInitialState() {
        return __awaiter(this, void 0, void 0, function* () {
            const initialState = yield this.serviceProxy.request(constants_1.GET_STATE_REQUEST, []);
            this.nextActionId = initialState.nextActionId;
            this.store.dispatch(setInitialState(initialState.state));
        });
    }
}
//# sourceMappingURL=vsls-redux.js.map